/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.setup.junit.internal;

import java.io.File;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubMonitor;

import com.ibm.team.build.client.ClientFactory;
import com.ibm.team.build.client.ITeamBuildClient;
import com.ibm.team.build.common.BuildItemFactory;
import com.ibm.team.build.common.builddefinition.BuildConfigurationRegistry;
import com.ibm.team.build.common.builddefinition.IAntConfigurationElement;
import com.ibm.team.build.common.builddefinition.IBuildDefinitionTemplate;
import com.ibm.team.build.common.builddefinition.IBuildTemplates;
import com.ibm.team.build.common.model.IBuildConfigurationElement;
import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildEngine;
import com.ibm.team.build.common.model.IBuildProperty;
import com.ibm.team.build.common.model.IConfigurationProperty;
import com.ibm.team.build.internal.common.builddefinition.IJazzScmConfigurationElement;
import com.ibm.team.build.setup.junit.constants.JUnitBuildDefinition;
import com.ibm.team.build.setup.junit.constants.JUnitBuildEngine;
import com.ibm.team.filesystem.setup.junit.constants.JUnitWorkspaces;
import com.ibm.team.foundation.setup.client.IProjectSetupContribution;
import com.ibm.team.foundation.setup.client.ProcessDescription;
import com.ibm.team.foundation.setup.client.repository.ISetupContext;
import com.ibm.team.process.common.IProcessArea;
import com.ibm.team.process.setup.junit.constants.JUnitProjectArea;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.scm.client.IWorkspaceConnection;

/**
 * Build contribution for project setup.
 */
public class BuildContribution implements IProjectSetupContribution {

    private ISetupContext fContext;

    public void contributeToProcessSpec(ISetupContext setup, ProcessDescription processSpec, IProgressMonitor monitor)
            throws TeamRepositoryException {
        // Do nothing.
    }

    public void contributeArtifacts(ISetupContext setup, ProcessDescription processDescription, IProgressMonitor monitor)
            throws TeamRepositoryException {
        SubMonitor progress = SubMonitor.convert(monitor, Messages.BuildContribution_PROGRESS, 100);
        try {
            createBuildDefinitions(setup, progress.newChild(25));
        } catch (Exception e) {
            throw new TeamRepositoryException(e);
        } finally {
            monitor.done();
        }
    }

    protected void createBuildDefinitions(ISetupContext context, IProgressMonitor monitor)
            throws TeamRepositoryException {
        fContext = context;
        ITeamRepository repo = context.getTeamRepository();
        ITeamBuildClient buildClient = ClientFactory.getTeamBuildClient(repo);
        IProcessArea processArea = context.getArtifact(JUnitProjectArea.JUnit);
        IWorkspaceConnection buildWorkspace = context.getArtifact(JUnitWorkspaces.Build);

        if (processArea == null) {
            throw new TeamRepositoryException(NLS.bind(Messages.BuildContribution_NO_TEAM_AREA, "JUnitTeamArea.Junit")); //$NON-NLS-1$
        }

        if (buildWorkspace == null) {
            throw new TeamRepositoryException(
                    NLS.bind(Messages.BuildContribution_NO_WORKSPACE, "JUnitWorkspaces.Build")); //$NON-NLS-1$
        }

        IBuildDefinition buildDefinition = buildClient.getBuildDefinition(JUnitBuildDefinition.JUnit.getId(), monitor);
        if (buildDefinition == null) {
            buildDefinition = BuildItemFactory.createBuildDefinition(processArea);
            buildDefinition.setId(decorateForTVT(JUnitBuildDefinition.JUnit.getId()));
            buildDefinition.setIgnoreWarnings(true);

            IBuildDefinitionTemplate template = BuildConfigurationRegistry.getInstance().getBuildDefinitionTemplate(
                    IBuildTemplates.TEMPLATE_ID_ANT);
            buildDefinition.initializeConfiguration(template);

            IBuildConfigurationElement jazzScmElement = BuildConfigurationRegistry.getInstance().getBuildConfigurationElement(
                    IJazzScmConfigurationElement.ELEMENT_ID);
            buildDefinition.initializeConfiguration(jazzScmElement);

            // setup properties
            IBuildConfigurationElement antElement = buildDefinition.getConfigurationElement(IAntConfigurationElement.ELEMENT_ID);

            String fetchDestination = "sandbox"; //$NON-NLS-1$
            String buildFilePath = fetchDestination + File.separator + "releng" + File.separator + "build.xml"; //$NON-NLS-1$ //$NON-NLS-2$

            addProperty(antElement, IAntConfigurationElement.PROPERTY_BUILD_FILE, buildFilePath);
            addProperty(antElement, IAntConfigurationElement.PROPERTY_ANT_ARGS, "-lib " + fetchDestination //$NON-NLS-1$
                    + File.separator + "releng" + File.separator + "compiler" + File.separator + "ecj.jar"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

            addProperty(buildDefinition, IJazzScmConfigurationElement.PROPERTY_FETCH_DESTINATION, fetchDestination);

            addProperty(buildDefinition, IJazzScmConfigurationElement.PROPERTY_WORKSPACE_UUID,
                    buildWorkspace.getResolvedWorkspace().getItemId().getUuidValue());

            addProperty(buildDefinition, IJazzScmConfigurationElement.PROPERTY_ACCEPT_BEFORE_FETCH, "true"); //$NON-NLS-1$

            addProperty(buildDefinition, IJazzScmConfigurationElement.PROPERTY_DELETE_DESTINATION_BEFORE_FETCH, "true"); //$NON-NLS-1$

            addProperty(buildDefinition, "buildLabelPrefix", "I", true); //$NON-NLS-1$ //$NON-NLS-2$

            buildClient.save(buildDefinition, monitor);
        }

        context.registerArtifact(JUnitBuildDefinition.JUnit, buildDefinition);

        // create build engine
        IBuildEngine buildEngine = buildClient.getBuildEngine(JUnitBuildEngine.JUnit.getId(), monitor);
        if (buildEngine == null) {
            buildEngine = BuildItemFactory.createBuildEngine(processArea);
            buildEngine.setId(decorateForTVT(JUnitBuildEngine.JUnit.getId()));
            buildEngine.setActive(true);
            buildEngine.setMonitoringThreshold(3);
            buildEngine.getSupportedBuildDefinitions().add(buildDefinition);
            buildClient.save(buildEngine, monitor);
        }

        context.registerArtifact(JUnitBuildEngine.JUnit, buildEngine);
    }

    private void addProperty(IBuildConfigurationElement element, String name, String value) {
        IConfigurationProperty property = element.getConfigurationProperty(name);
        if (property == null) {
            property = BuildItemFactory.createConfigurationProperty();
            property.setName(name);
            element.getConfigurationProperties().add(property);
        }
        property.setValue(value);
    }

    private void addProperty(IBuildDefinition buildDefinition, String name, String value, boolean isGenericEditAllowed) {
        IBuildProperty property = buildDefinition.getProperty(name);
        if (property == null) {
            property = BuildItemFactory.createBuildProperty();
            property.setName(name);
            property.setGenericEditAllowed(isGenericEditAllowed);
            buildDefinition.getProperties().add(property);
        }
        property.setValue(value);
    }

    private void addProperty(IBuildDefinition buildDefinition, String name, String value) {
        addProperty(buildDefinition, name, value, false);
    }

    private String decorateForTVT(String input) {
        if (!fContext.createForTVTUse())
            return input;
        return "ZZZ" + input + "ZZZ"; //$NON-NLS-1$//$NON-NLS-2$
    }

}
